// Utility functions useful for tests

// Copyright (C) 2021  Simon Ruderich
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package testutil

import (
	"fmt"
	"reflect"
	"testing"

	"github.com/google/go-cmp/cmp"
)

func AssertEqual(t *testing.T, name string, act, exp interface{}) {
	t.Helper()

	if !reflect.DeepEqual(act, exp) {
		t.Errorf("%s: %s", name, cmp.Diff(exp, act))
	}
}

func AssertErrorEqual(t *testing.T, name string, act, exp error) {
	t.Helper()

	// Ugly but the simplest way to compare errors (including nil)
	actStr := fmt.Sprintf("%s", act)
	expStr := fmt.Sprintf("%s", exp)
	if actStr != expStr {
		t.Errorf("err = %s (%#v), want %s (%#v)",
			actStr, act, expStr, exp)
	}
}
